@@ -8,7 +8,27 @@ import xattr
 
 
 class BaseTestXattr(object):
-    def test_attr(self):
+    # TESTDIR for temporary files usually defaults to "/tmp",
+    # which may not have XATTR support (e.g. tmpfs);
+    # manual override here.
+    TESTDIR = None
+
+    def test_attr_fs_encoding_unicode(self):
+        # Not using setlocale(LC_ALL, ..) to set locale because
+        # sys.getfilesystemencoding() implementation falls back
+        # to user's preferred locale by calling setlocale(LC_ALL, '').
+        xattr.compat.fs_encoding = 'UTF-8'
+        self._test_attr()
+
+    def test_attr_fs_encoding_ascii(self):
+        xattr.compat.fs_encoding = 'US-ASCII'
+        if sys.version_info[0] < 3:
+            with self.assertRaises(UnicodeEncodeError):
+                self._test_attr()
+        else:
+            self._test_attr()
+
+    def _test_attr(self):
         x = xattr.xattr(self.tempfile)
 
         # Solaris 11 and forward contain system attributes (file flags) in
@@ -19,6 +39,10 @@ class BaseTestXattr(object):
             d['SUNWattr_ro'] = x['SUNWattr_ro']
             d['SUNWattr_rw'] = x['SUNWattr_rw']
 
+        # SELinux systems use an attribute which must be accounted for
+        if sys.platform.startswith('linux') and 'security.selinux' in x:
+            d['security.selinux'] = x['security.selinux']
+
         self.assertEqual(list(x.keys()), list(d.keys()))
         self.assertEqual(list(x.list()), list(d.keys()))
         self.assertEqual(dict(x), d)
@@ -66,15 +90,17 @@ class BaseTestXattr(object):
             self.assertEqual(str(e), msg)
 
     def test_symlink_attrs(self):
-        # Solaris doesn't support extended attributes on symlinks
-        if sys.platform == 'sunos5':
-            return
         symlinkPath = self.tempfilename + '.link'
         os.symlink(self.tempfilename, symlinkPath)
         try:
             symlink = xattr.xattr(symlinkPath, options=xattr.XATTR_NOFOLLOW)
             realfile = xattr.xattr(self.tempfilename)
-            symlink['user.islink'] = b'true'
+            try:
+                symlink['user.islink'] = b'true'
+            except IOError:
+                # Solaris, Linux don't support extended attributes on symlinks
+                raise unittest.SkipTest("XATTRs on symlink not allowed"
+                                        " on filesystem/platform")
             self.assertEqual(dict(realfile), {})
             self.assertEqual(symlink['user.islink'], b'true')
         finally:
@@ -83,7 +109,7 @@ class BaseTestXattr(object):
 
 class TestFile(TestCase, BaseTestXattr):
     def setUp(self):
-        self.tempfile = NamedTemporaryFile()
+        self.tempfile = NamedTemporaryFile(dir=self.TESTDIR)
         self.tempfilename = self.tempfile.name
 
     def tearDown(self):
@@ -92,7 +118,7 @@ class TestFile(TestCase, BaseTestXattr):
 
 class TestDir(TestCase, BaseTestXattr):
     def setUp(self):
-        self.tempfile = mkdtemp()
+        self.tempfile = mkdtemp(dir=self.TESTDIR)
         self.tempfilename = self.tempfile
 
     def tearDown(self):
@@ -107,7 +133,9 @@ except AttributeError:
 else:
     class TestFileWithSurrogates(TestFile):
         def setUp(self):
-            if sys.platform != 'linux':
+            if sys.platform not in ('linux', 'linux2'):
                 raise unittest.SkipTest('Files with invalid encoded names are only supported under linux')
-            self.tempfile = NamedTemporaryFile(prefix=b'invalid-\xe9'.decode('utf8','surrogateescape'))
+            if sys.version_info[0] < 3:
+                raise unittest.SkipTest('Test is only available on Python3') # surrogateescape not avail in py2
+            self.tempfile = NamedTemporaryFile(prefix=b'invalid-\xe9'.decode('utf8','surrogateescape'), dir=self.TESTDIR)
             self.tempfilename = self.tempfile.name
